/*
 * Copyright (C) 2012 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* =====================================================================================================
 * The UI Activity of the application that connects a Bluetooth accessory to and Android phone and
 * displays information on battery, missed calls and unread SMS messages
 * The Activity activates the service coded in PhoneInfoServer.java that keeps running all the time
 * =====================================================================================================
 * By: Zakie Mashiah 
 * =====================================================================================================
 */

package myPhoneInfo.test.zakiem;


import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

/**
 * The main activity
 **
 */
public class PhoneInfoActivity extends Activity {
	
	private static final String TAG = "PhoneInfoActivity";
	public  static final String BROADCAST_SET_DISPLAY_INTENTSITY = "myPhoneInfo.test.zakiem.SET_DISPLAY_INTENSITY";
	public  static final String DISPLAY_INTENSITY = "DisplayIntensity";

	// Intent request codes for DeviceList Activity
    private static final int REQUEST_ENABLE_BT = 3;

    // Message types sent from the PhoneInfoServer Handler
    public static final int MESSAGE_STATE_CHANGE = 1;
    public static final int MESSAGE_READ = 2;
    public static final int MESSAGE_WRITE = 3;
    public static final int MESSAGE_DEVICE_NAME = 4;
    public static final int MESSAGE_TOAST = 5;
    public static final int MESSAGE_REFRESHSCREEN = 6;
    private BluetoothAdapter mBTAdapter = null;
    private PhoneInfoDataOnly currentData;
    private int displayIntensity = 8;

    // Name of the connected device
    private String mConnectedDeviceName = null;

    private Intent mServiceIntent;
    private Intent mTalkToService;
    
    // The different UI elements
	private TextView mBatteryView;
	private TextView mUnreadSMSView;
	private TextView mMissedCallsView;
	private SeekBar mDisplayInensitySeekBar;
	private ImageView mBtStatus;

    // The Handler that gets information back from the PhoneInfoServer
    public class FromServiceBR extends BroadcastReceiver{
        @Override public void onReceive(Context context, Intent intent) {
        	String action = intent.getAction();
        	Bundle bundle;
        	if (action.equals(PhoneInfoServer.BROADCAST_ACCESORY_DEVICE_NAME)) {
        		bundle = intent.getBundleExtra(PhoneInfoServer.DEVICE_NAME);
        		String dn = bundle.getString(PhoneInfoServer.DEVICE_NAME);
        		if (dn != null) {
        			mConnectedDeviceName = dn;
        			if (mConnectedDeviceName == null)
        				dn = "unknown";
        			Log.d(TAG, "Setting device name to: " + dn);
        			mBtStatus.setImageResource(R.drawable.ic_btc); // If we are getting device name, we are connected
        			Toast.makeText(getApplicationContext(), "Connected to: " + dn, Toast.LENGTH_SHORT).show();
        		}
        		else
        			Log.d(TAG, "Error setting device name: No device name");
        	}
        	else if (action.equals(PhoneInfoServer.BROADCAST_DISPLAY_INTENSITY)) {
        		bundle = intent.getBundleExtra(PhoneInfoServer.DISPLAY_INTENSITY);
        		displayIntensity = bundle.getInt(PhoneInfoServer.DISPLAY_INTENSITY);
        		mDisplayInensitySeekBar.setProgress(displayIntensity);	// Update the seek-bar control
        		Log.d(TAG, "Device current intensity is:" + displayIntensity);
    			mBtStatus.setImageResource(R.drawable.ic_btc); // If we are getting device name, we are connected
        	}
        	else if (action.equals(PhoneInfoServer.BROADCAST_PHONE_BASIC_INFO)) {
        		bundle = intent.getBundleExtra(PhoneInfoServer.PHONE_BASIC_INFO);
        		PhoneInfoDataOnly newData = new PhoneInfoDataOnly();
            	newData.set(bundle);
            	if (currentData.equal(newData) == false ) {
            		currentData.set(newData);
            		Log.d(TAG, "refreshing screen");
        			mBtStatus.setImageResource(R.drawable.ic_btc); // If we are getting device name, we are connected
            		showOnScreen();
            	}
            	else
            		Log.d(TAG, "New information received. No change");
        	}
        	else if (action.equals(PhoneInfoServer.BROADCAST_STATE_CHANGE)) {
        		bundle = intent.getBundleExtra(PhoneInfoServer.STATE_CHANGE);
        		int newstate = bundle.getInt(PhoneInfoServer.STATE_CHANGE);
        		switch (newstate) {
        		case PhoneInfoServer.STATE_CONNECTED:
        			Log.d(TAG, "Handler: state connected");
                	mBtStatus.setImageResource(R.drawable.ic_btc);
                	String dn = (mConnectedDeviceName == null) ? "unknown" : mConnectedDeviceName;
                	Toast.makeText(getApplicationContext(), dn + " is now connected", Toast.LENGTH_SHORT).show();
                    break;
                case PhoneInfoServer.STATE_CONNECTING:
                	Log.d(TAG, "Handler: state connecting");
                	mBtStatus.setImageResource(R.drawable.ic_btnc);
                	mConnectedDeviceName = null;
                    break;
                case PhoneInfoServer.STATE_LISTEN:
                	Log.d(TAG, "Handler: state listen");
                	mBtStatus.setImageResource(R.drawable.ic_btnc);
                	mConnectedDeviceName = null;
                    break;
                case PhoneInfoServer.STATE_NONE:
                	Log.d(TAG, "Handler: state none");
                	mBtStatus.setImageResource(R.drawable.ic_btnc);
                	mConnectedDeviceName = null;
                    break;
                }
        	}
        	else if (action.equals(PhoneInfoServer.BROADCAST_TOAST)) {
        		bundle = intent.getBundleExtra(PhoneInfoServer.TOAST);
        		String message = bundle.getString(PhoneInfoServer.TOAST);
        		if (message != null) {
        			Toast.makeText(getApplicationContext(), message, Toast.LENGTH_LONG).show();
        			Log.d(TAG, "Showing toast message " + message);
        		}
        		else
        			Log.d(TAG, "null message to toast, ignoring");
        	}
        	else
        		Log.e(TAG, "Unknown broadcast message received");
        }
    };
    private FromServiceBR mServiceBR = new FromServiceBR();


    /* =======================================================================================
     * The Activity lifecycle functions
     * =======================================================================================
     */
    
    @Override public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.phone_info_layout);
        mBtStatus = (ImageView)this.findViewById(R.id.ic_bt_status);

        Log.i(TAG, "Starting PhoneInfo Activity");
       
        // Set the BT adapter
        mBTAdapter = BluetoothAdapter.getDefaultAdapter();
        if (mBTAdapter == null) {
        	Toast.makeText(this, "Bluetooth is not available", Toast.LENGTH_LONG).show();
        	finish();
        	return;
        }

        // Setup the data and views/layout
        currentData = new PhoneInfoDataOnly();
        setViews();

        // Set the service intent and broadcast receiver
        mServiceIntent = new Intent(this, PhoneInfoServer.class);
        mTalkToService = new Intent(PhoneInfoActivity.BROADCAST_SET_DISPLAY_INTENTSITY);
    }

    // Function to be called onCreate to set all the views of the layout for UI
    public void setViews() {
		mBatteryView = (TextView)findViewById(R.id.batteryInfo);
		mUnreadSMSView = (TextView)findViewById(R.id.unreadSMSInfo);
		mMissedCallsView = (TextView)findViewById(R.id.missedCallsInfo);
		mDisplayInensitySeekBar = (SeekBar)findViewById(R.id.displayInentisty);
		mDisplayInensitySeekBar.setProgress(displayIntensity);
		mDisplayInensitySeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener()
		{
			public void onProgressChanged(SeekBar seekBar, int progress, boolean fromTouch)
			{
				displayIntensity = (progress < 0) ? 0 : (progress > 15) ? 15 : progress; // value between 0 to 15 (0x00 - 0x0F)
				mTalkToService.putExtra(DISPLAY_INTENSITY, displayIntensity);
				sendBroadcast(mTalkToService);
			}
			public void onStopTrackingTouch(SeekBar seekBar) {}
			public void onStartTrackingTouch(SeekBar seekBar)
			{
				seekBar.setProgress(displayIntensity);
			}
		}
		); 
    }
    
    
    @Override protected void onStart() {
        super.onStart();

        Log.d(TAG, "onStart()");
        if (mBTAdapter.isEnabled() == false) {
        	Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
        	startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
        }
        else {
        	Log.d(TAG, "Starting phoneInfo");
        	//if (phoneInfo.mInitializedAndRunning == false) // Only once
        		startService(mServiceIntent);
        }
    }
    
    
    
    private void showOnScreen() {
    	PhoneInfoDataOnly d = currentData;
    	String status = "Battery: ";
        status += (d.batteryFull) ? "Full" : Integer.toString(d.batteryPercentile) + "%";
        mBatteryView.setText(status);

        status = "Unread SMS: ";
        status += Integer.toString(d.unreadSMS);
        mUnreadSMSView.setText(status);

        status = "Missed Calls: ";
        status += Integer.toString(d.missedCalls);
        mMissedCallsView.setText(status);
    }
    
        
    @Override public void onDestroy() {
    	Log.d(TAG, "onDestroy");
    	if (mServiceBR != null)
    		unregisterReceiver(mServiceBR);
    	super.onDestroy();
    }

        
    @Override protected void onRestart() {
        super.onRestart();
        Log.d(TAG, "onRestart");
    }

    
    @Override public void onResume() {
        super.onResume();
        Log.d(TAG, "onResume()");
        IntentFilter ifFromService = new IntentFilter();
        ifFromService.addAction(PhoneInfoServer.BROADCAST_ACCESORY_DEVICE_NAME);
        ifFromService.addAction(PhoneInfoServer.BROADCAST_DISPLAY_INTENSITY);
        ifFromService.addAction(PhoneInfoServer.BROADCAST_PHONE_BASIC_INFO);
        ifFromService.addAction(PhoneInfoServer.BROADCAST_STATE_CHANGE);
        ifFromService.addAction(PhoneInfoServer.BROADCAST_TOAST);
        this.registerReceiver(mServiceBR, ifFromService);
        showOnScreen();
    }
    
    /* =======================================================================================
     * Menu constants and code
     * =======================================================================================
     */
    private static final String mExitMenu = "Exit";
    private static final String mShowMACMenu = "Show MAC";
    
    @Override public boolean onCreateOptionsMenu(Menu menu) {
    	menu.add(mShowMACMenu);
		menu.add(mExitMenu);
		return true;
	}

    
    @Override public boolean onOptionsItemSelected(MenuItem item) {
    	if (item.getTitle() == mShowMACMenu) {
    		if (mBTAdapter != null) {
    			String s = mBTAdapter.getAddress();
    			if (s != null)
    				Toast.makeText(getApplicationContext(), s, Toast.LENGTH_LONG).show();
    		}
    	}
    	else if (item.getTitle() == mExitMenu) {
			finish();
			stopService(mServiceIntent);
			System.exit(0);
		}
		return true;
	}
 }
