/* =====================================================================================================
 * The background service for communicating over Bluetooth with the accessory and listens on system
 * events related to Battery status, Incoming SMS messages and missed calls
 * Threading and listening on BT is taken from the Android developers sample code of BluetoothChat,
 * extended here to include registration of system events of intrest to us as well as back and forth
 * communication with the Activity UI
 * =====================================================================================================
 * By: Zakie Mashiah 
 * =====================================================================================================
 */
package myPhoneInfo.test.zakiem;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.Uri;
import android.os.BatteryManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.provider.CallLog.Calls;
import android.util.Log;



public class PhoneInfoServer extends Service {

	/* Variables and constants
	 * ============================================================
	 */
	private static final String TAG = "PhoneInfoServer";
	// The messages it will get in broadcast
	public  static final String BROADCAST_PHONE_BASIC_INFO = "myPhoneInfo.test.zakiem.PHONE_BASIC_INFO";
	public  static final String PHONE_BASIC_INFO = "BasicInfo";
	public  static final String BROADCAST_ACCESORY_DEVICE_NAME = "myPhoneInfo.test.zakiem.DEVICE_NAME";
	public  static final String DEVICE_NAME = "DeviceName";
	public  static final String BROADCAST_TOAST = "myPhoneInfo.test.zakiem.TOAST";
	public  static final String TOAST = "Toast";
	public  static final String BROADCAST_STATE_CHANGE = "myPhoneInfo.test.zakiem.STATE_CHANGE";
	public  static final String STATE_CHANGE = "StateChange";
    // Constants that indicate the current connection state
    public static final int STATE_NONE = 0;       // we're doing nothing
    public static final int STATE_LISTEN = 1;     // now listening for incoming connections
    public static final int STATE_CONNECTING = 2; // now initiating an outgoing connection
    public static final int STATE_CONNECTED = 3;  // now connected to a remote device
    public static final String BROADCAST_DISPLAY_INTENSITY = "myPhoneInfo.test.zakiem.DISPLAY_INTENSITY";
    public static final String DISPLAY_INTENSITY = "DisplayIntensity"; // The current intensity value in the display

	// and the associated intents
	private Intent mPhoneBasicInfoIntent;
	private Intent mAccesoryDevicNameIntent;
	private Intent mToastIntent;
	private Intent mStateChangeIntent;
	private Intent mDisplayIntensityIntent;
	
	// The data of the phone we are interested in
	private PhoneInfoDataOnly currentData;		// The data we have on the phone and want to display
	private PhoneInfoDataOnly lastData;			// The data we sent last time
	private Handler batHandler = new Handler();	// Handler for battery broadcasts

	
	// BT members and constants
    private static final UUID BT_SPP_UUID = UUID.fromString("00001101-0000-1000-8000-00805f9b34fb"); // SPP UID
    private static final String NAME_INSECURE = "PhoneInfoAcc"; // Name for the SDP record when creating server socket
	private BluetoothAdapter mAdapter;
    private AcceptThread mInsecureAcceptThread;
    private ConnectedThread mConnectedThread;
    private int mState;		// See STATE_xxx constants above

    // Constants used by ContentObservers (happy Contento) to notify us of changes
    public static final int CONTENTO_INFOCHANGED = 0x1010;
    public static final int CONTENTO_MC = 0x1; // Missed Calls
    public static final int CONTENTO_US = 0x2; // Unread SMS
    
    // Notification to system notification area
    private NotificationManager mNotificationMgr;
	private CharSequence mNotificationTickerTitle = "PhoneInfo";
	private Notification mNotification = new Notification(R.drawable.bt7seg, mNotificationTickerTitle, System.currentTimeMillis());
	private Intent mNotificationIntent;
	private PendingIntent mNotificationPI;
	private static int PHONE_INFO_NOTIFICATION = 2266; // My birthday 2/2/1966

	// Handler to listen to events from the content observes of SMS/Calls
    private final Handler mContentObserversHandler = new Handler() {
        @Override public void handleMessage(Message msg) {
        	Log.d(TAG, "Handling message " + msg.what + " from " + msg.arg2);
            switch (msg.what) {
            case CONTENTO_INFOCHANGED:
            	sendToBTAccessory();
            	sendDataToActivity();
            	break;
            }
        }
    };

    // Broadcast receiver for system battery notifications
    private BroadcastReceiver myBatteryReceiver = new BroadcastReceiver() {
		@Override public void onReceive(Context context, Intent intent) {
			if (intent.getAction().equals(Intent.ACTION_BATTERY_CHANGED)) {
				currentData.batteryPercentile = intent.getIntExtra("level", 0);
				currentData.batteryFull = (intent.getIntExtra("status", -1) == BatteryManager.BATTERY_STATUS_FULL);
				batHandler.post(new Runnable() { 
					public void run() {
						Log.d(TAG, "Battery onReceive broadcast receiver");
						// Could not find a way to get notified once SMS message is read,
						// so using battery receiver to poll for SMS data.
						// The other alternative was to periodically do that, but that is
						// actually adding more code and CPU consumption for no good reason
						// Battery apparently broadcasts itself every 30 seconds, so that
						// is good enough for us.
						currentData.unreadSMS = mUnreadSMSContentObserver.getUnreadSMS();
						sendToBTAccessory();
						sendDataToActivity();
					}
				} );
			}
		}
	};
	
	// Broadcast receiver for events coming from the UI activity
	public class FromActivityBR  extends BroadcastReceiver {
		@Override public void onReceive(Context context, Intent intent) {
			if (intent.getAction().equals(PhoneInfoActivity.BROADCAST_SET_DISPLAY_INTENTSITY)) {
				int intensity = intent.getIntExtra(PhoneInfoActivity.DISPLAY_INTENSITY, 8);
				setDisplayIntensity(intensity);
			}
			else
				Log.d(TAG, "Unknown action received from activity");
		}
	};
	FromActivityBR mFromActivityBR = new FromActivityBR();
	
	// The content overserver
	private ContentResolver mContentResolver;
    private MissedCallsContentObserver mMissedCallsContentOberserver;
    private UnreadSMSContentObserver mUnreadSMSContentObserver;
    private boolean mContentObserversRegistered = false;

    // Data from Accessory reply field each time the service connects to Accessory
    private AccessoryInfo mAccessoryInfo = new AccessoryInfo();
    
    /* Constructor and service life cycle methods
     * ============================================================
     * */
    public PhoneInfoServer() {	}


    @Override public void onCreate() {
    	// Setup the different intents
    	mPhoneBasicInfoIntent 		= new Intent(BROADCAST_PHONE_BASIC_INFO);
    	mAccesoryDevicNameIntent 	= new Intent(BROADCAST_ACCESORY_DEVICE_NAME);
    	mToastIntent 				= new Intent(BROADCAST_TOAST);
    	mStateChangeIntent 			= new Intent(BROADCAST_STATE_CHANGE);
    	mDisplayIntensityIntent 	= new Intent(BROADCAST_DISPLAY_INTENSITY);
    	// The Bluetooth stuff
    	mAdapter = BluetoothAdapter.getDefaultAdapter(); 
		mState = STATE_NONE;
		// our data
		currentData = new PhoneInfoDataOnly();
		// Content observers
		mContentResolver = this.getContentResolver();
		mMissedCallsContentOberserver = new MissedCallsContentObserver(mContentResolver, mContentObserversHandler);
		mUnreadSMSContentObserver = new UnreadSMSContentObserver(mContentResolver, mContentObserversHandler);
		mNotificationMgr = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
    }
    
    @Override public int onStartCommand(Intent intent, int flags, int startId) {
    	this.registerReceiver(mFromActivityBR, new IntentFilter(PhoneInfoActivity.BROADCAST_SET_DISPLAY_INTENTSITY));
    	start();
    	
    	// Make ourselves a foreground service, otherwise the system will kill us after a while
    	mNotificationIntent  = new Intent(this, PhoneInfoActivity.class);
    	mNotificationIntent.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT | Intent.FLAG_ACTIVITY_SINGLE_TOP);
    	mNotificationPI = PendingIntent.getActivity(this, 0, mNotificationIntent, 0);
    	mNotification.setLatestEventInfo(this, mNotificationTickerTitle, "Idle", mNotificationPI); 
    	mNotification.flags |= Notification.FLAG_NO_CLEAR;
    	startForeground(PHONE_INFO_NOTIFICATION, mNotification);
    	
    	return START_STICKY;
    }
    
    @Override public IBinder onBind(Intent intent) {
        // We don't provide binding, so return null
        return null;
    }
    
    @Override public void onDestroy() {
      Log.d(TAG, "onDestroy");
      stop();
      if (mFromActivityBR != null) {
    	  unregisterReceiver(mFromActivityBR);
    	  mFromActivityBR = null;
      }
      if (mContentResolver != null) {
    	  mContentResolver.unregisterContentObserver(mUnreadSMSContentObserver);
    	  mContentResolver.unregisterContentObserver(mMissedCallsContentOberserver);
    	  mUnreadSMSContentObserver = null;
    	  mMissedCallsContentOberserver = null;
      }
      stopForeground(true);
      stopSelf();
    }
    
    
    /* Notification handling for new connection and disconnection
     * ============================================================
     */
    private void sendBarNotification() {

    	Context context = this.getApplicationContext();
    	CharSequence content;
    	if (mState == STATE_CONNECTED)
    		content = "Connected to: " + mAccessoryInfo.getAccessoryType();
    	else
    		content = "Disconnected";
    	
    	mNotificationMgr.cancelAll();
    	mNotification.setLatestEventInfo(context, mNotificationTickerTitle, content, mNotificationPI);
    	mNotificationMgr.notify(PHONE_INFO_NOTIFICATION, mNotification);
    }
    
    
    /* Methods to communicate with parent activity
     * ============================================================
     */
    /** Sends a string to Activity using broadcast
     * @param i	The intent of this broadcast
     * @param s The string to send
     * @param tokenName	the name of the token to use inside the bundle
     */
    private void _sendStringToActivity(Intent i, String s, String tokenName) {
    	Bundle bundle = new Bundle();
    	bundle.putString(tokenName, s);
    	i.putExtra(tokenName, bundle);
    	sendBroadcast(i);
    }
    
    /** Sends an integer to Activity using broadcast
     * @param	i	The intent of this broadcast
     * @param	num	The integer to send
     * @param	tokenName	The name of the token to use inside the bundle
     */
    private void _sendIntToActivity(Intent i, int num, String tokenName) {
    	Bundle bundle = new Bundle();
    	bundle.putInt(tokenName, num);
    	i.putExtra(tokenName, bundle);
    	sendBroadcast(i);
    }
    
    // Sends the entire PhoneInfoDataOnly class to Activity
    private void sendDataToActivity() {
    	Log.d(TAG, "Sending data to activity");
        Bundle bundle = currentData.get();
        mPhoneBasicInfoIntent.putExtra(PHONE_BASIC_INFO, bundle);
        sendBroadcast(mPhoneBasicInfoIntent);
    }
    
    private void sendToastToActivity(String s) {
    	Log.d(TAG, "Sending toast to activity");
    	_sendStringToActivity(mToastIntent, s, TOAST);
    }
    
    // The bluetooth device name not the accessory name
    private void sendDeviceNameToActivity(String deviceName) {
    	Log.d(TAG, "Sending device name to activity");
    	_sendStringToActivity(mAccesoryDevicNameIntent, deviceName, DEVICE_NAME);
    }
    
    private void sendStateChangedToActivity() {
    	Log.d(TAG, "Sending state change to activity");
    	_sendIntToActivity(mStateChangeIntent, mState, STATE_CHANGE);
    }
    
    private void sendDisplayIntensityToActivity(int intensity) {
    	_sendIntToActivity(mDisplayIntensityIntent, intensity, DISPLAY_INTENSITY);
    }


    /* Methods to communicate with the BT device
     * =============================================== 
     */
    public void setDisplayIntensity(int di) {
    	Log.d(TAG, "Setting intensity to " + di);
    	
    	ConnectedThread r;
    	
    	synchronized(this) {
			if (mState != STATE_CONNECTED)
				return;
			r = mConnectedThread;
		}
    	String s = "I" + Integer.toHexString(di) + '\r';
    	Log.d(TAG, "Sending " + s + " to BT accessory");
		r.write(s.getBytes());
    }

    // Sends the battery percentile/full missed calls and unread SMS data to accessory over BT
    private void sendToBTAccessory() {
		ConnectedThread r;
		
		synchronized(this) {
			if (mState != STATE_CONNECTED)
				return;
			r = mConnectedThread;
		}
		if (currentData == null) {
			Log.e(TAG, "send() does not have any data");
			return;
		}

		
		// Fetch the values from the content observers
		currentData.missedCalls = mMissedCallsContentOberserver.missedCalls;
		currentData.unreadSMS = mUnreadSMSContentObserver.unreadSMS;

		// if no change in data, do not send (preserving battery power)
		if (lastData != null) // do we have last data?
			if (lastData.equal(currentData))
				return; // no change
			else // set to new values
				lastData.set(currentData);
		else // create last data
			lastData = new PhoneInfoDataOnly(currentData.missedCalls, currentData.unreadSMS, currentData.batteryPercentile, currentData.batteryFull);

		// Build the string to send, now you tell me isn't sprintf a more elegant way?
		String s = "B" + currentData.batteryPercentile + "/";
		s += currentData.batteryFull ? "1" : "0";
		s += "S" + currentData.unreadSMS;
		s += "C" + currentData.missedCalls + "\r";

		Log.d(TAG, "Sending " + s + "to BT accessory");
		// write through the writing thread
		r.write(s.getBytes());
	}
	

    // Sends 'i' to the accessory, which should result with information about the accessory
    private void queryBTAccessory() {
    	ConnectedThread r;
		
		synchronized(this) {
			if (mState != STATE_CONNECTED)
				return;
			r = mConnectedThread;
		}

		Log.d(TAG, "Query BT accessory");
		String q = new String("i\r");
		// write through the writing thread
		r.write(q.getBytes());
    }

    
    /*
     * State get and set methods to handle state changes
     * ============================================================
     */
    private synchronized void setState(int state) {
        mState = state;

        Log.d(TAG, "Setting state to " + state);
        sendStateChangedToActivity();
    }

    // Return the current connection state.
    public synchronized int getState() {
        return mState;
    }
    

    /*
     * Stating and stopping the service (internally to my class)
     * ============================================================
     */
     // Start the chat service. Specifically start AcceptThread to begin a
     // session in listening (server) mode. Called by service lifecycle method onStartCommand
    public synchronized void start() {
    	Log.d(TAG, "start() staring");
    	if (mState == STATE_CONNECTED) // If we do not do that, accessory will disconnect and connect again on application resume
    		return;
    	
    	Log.d(TAG, "start() cleaning threads");
        // Cancel any thread currently running a connection
        if (mConnectedThread != null) {
        	mConnectedThread.cancel();
        	mConnectedThread = null;
        }

        // Register the content observers 
    	lastData = null;
    	if (mContentObserversRegistered == false) { // Only once
    		if (mContentResolver != null) {
    			currentData.unreadSMS = mUnreadSMSContentObserver.getUnreadSMS();
    			currentData.missedCalls = mMissedCallsContentOberserver.getMissedCalls();
    			
    			Log.d(TAG, "PhoneInfo.start registering content observers");
    			mContentResolver.registerContentObserver(Calls.CONTENT_URI, true, mMissedCallsContentOberserver);
    			mContentResolver.registerContentObserver(Uri.parse("content://sms/"), true, mUnreadSMSContentObserver);
    			mContentResolver.registerContentObserver(Uri.parse("content://sms/inbox/"), true, mUnreadSMSContentObserver);
    	        // Put the Battery callbacks
    			this.registerReceiver(PhoneInfoServer.this.myBatteryReceiver, new IntentFilter(Intent.ACTION_BATTERY_CHANGED));
    			Log.d(TAG, "PhoneInfo.start done registration");
    			mContentObserversRegistered = true;
    		}
    	}

        Log.d(TAG, "start() creating AcceptThread");
        setState(STATE_LISTEN);
        if (mInsecureAcceptThread == null) {
            mInsecureAcceptThread = new AcceptThread(false);
            mInsecureAcceptThread.start();
        }
        
        Log.d(TAG, "start() complete.");
    } 
    
    /**
     * Start the ConnectedThread to begin managing a Bluetooth connection
     * @param socket  The BluetoothSocket on which the connection was made
     * @param device  The BluetoothDevice that has been connected
     */
    public synchronized void connected(BluetoothSocket socket, BluetoothDevice
            device, final String socketType) {

    	Log.d(TAG, "PhoneInfo.connected(): socket type:" + socketType + "Canceling other threads");

        // Cancel any thread currently running a connection
        if (mConnectedThread != null) {
        	mConnectedThread.cancel();
        	mConnectedThread = null;
        }

        // Cancel the accept thread because we only want to connect to one device
        if (mInsecureAcceptThread != null) {
            mInsecureAcceptThread.cancel();
            mInsecureAcceptThread = null;
        }

        // Start the thread to manage the connection and perform transmissions
        Log.d(TAG, "PhoneInfo.connected(): Creating connected thread");
        mConnectedThread = new ConnectedThread(socket, socketType);
        mConnectedThread.start();

        // Send the name of the connected device back to the UI Activity
        Log.d(TAG, "PhoneInfo.connected(): Sending device name to calling activity");
        sendDeviceNameToActivity(device.getName());
        setState(STATE_CONNECTED);
        sendToBTAccessory();	// Start by sending data to accessory
        queryBTAccessory();		// Query the the accessory for its information
    }
    
    // Stop all threads
    public synchronized void stop() {

    	Log.d(TAG, "PhoneInfo.stop(): stop all threads");

        if (mConnectedThread != null) {
            mConnectedThread.cancel();
            mConnectedThread = null;
        }

        if (mInsecureAcceptThread != null) {
            mInsecureAcceptThread.cancel();
            mInsecureAcceptThread = null;
        }
        
        lastData = null;
        setState(STATE_NONE);
    } 
    
    // Indicate that the connection was lost and notify the UI Activity.
    private void connectionLost() {
        // Send a failure message back to the Activity
    	sendToastToActivity("Device connection was lost");
    	setState(STATE_NONE);
    	
    	sendBarNotification();

        // Start this service over to so that we get the accepting thread
    	// back again
        PhoneInfoServer.this.start();
    }
    
    // We had a problem doing Accept
    private void acceptProblem() {
        sendToastToActivity("Could not create BT service.\nTry disable and enable BT");
    }

/* =====================================================================================
 * The different types of threads we will be managing: AcceptThread to wait for the BT
 * accessory to connect to the phone abd ConnectedThread that will do actual work with
 * the accessory once connected
 * =====================================================================================    
 */
/**
 * This thread runs while listening for incoming connections. It behaves
 * like a server-side client. It runs until a connection is accepted
 * (or until cancelled).
 */
private class AcceptThread extends Thread {
    // The local server socket
	static private final String TAG = "PhoneInfoAcceptThread";
    private final BluetoothServerSocket mmServerSocket;
    private String mSocketType;


    /** Creates an thread for accepting incoming Bluetooth connections
     * @param secure	Currently ignored, but suppose to represent the mode of socket.
     * All communication is currently done over insecure socket 
     */
    public AcceptThread(boolean secure) {
        BluetoothServerSocket tmp = null;
        mSocketType = secure ? "Secure":"Insecure";

        // Create a new listening server socket
        try {
        	Log.d(TAG, "AcceptThread constructor trying to create listening socket");
            if (!secure)
            	// This is for Android 2.2
            	tmp = mAdapter.listenUsingRfcommWithServiceRecord(NAME_INSECURE, BT_SPP_UUID);
            	// This is for Android 2.3 but testing the above on 2.3 device showed it to be working.
                //tmp = mAdapter.listenUsingInsecureRfcommWithServiceRecord(NAME_INSECURE, BT_SPP_UUID);
            Log.d(TAG, "AcceptThread: Listening BT Socket " + mSocketType + " created");
        } catch (IOException e) {
            Log.e(TAG, "AcceptThread: Listening BT Socket Type: " + mSocketType + " listen() failed " + e.getMessage());
            acceptProblem();
        }
        mmServerSocket = tmp;
    }


    public void run() {
    	
    	if (mmServerSocket == null) {
    		Log.e(TAG, "AcceptThread.run: No server socket");
    		return;
    	}
    	Log.d(TAG, "AcceptThread.run: socket type:" + mSocketType);
        setName("AcceptThread" + mSocketType);

        BluetoothSocket socket = null;

        // Listen to the server socket if we're not connected
        while (mState != STATE_CONNECTED) {
            try {
                // This is a blocking call and will only return on a
                // successful connection or an exception
                socket = mmServerSocket.accept();
                Log.d(TAG, "AcceptThread.run: returned from accept");
            } catch (IOException e) {
                Log.e(TAG, "AcceptThread.run: Socket Type: " + mSocketType + "accept() failed " + e.getMessage());
                break;
            }

            // If a connection was accepted
            if (socket != null) {
                synchronized (PhoneInfoServer.this) {
                	Log.d(TAG, "AcceptThread.run: Got accept request");
                    switch (mState) {
                    case STATE_LISTEN:
                    case STATE_CONNECTING:
                        // Situation normal. Start the connected thread.
                        connected(socket, socket.getRemoteDevice(), mSocketType);
                        break;
                    case STATE_NONE:
                    case STATE_CONNECTED:
                        // Either not ready or already connected. Terminate new socket.
                        try {
                            socket.close();
                        } catch (IOException e) {
                            Log.e(TAG, "Could not close unwanted socket" + e.getMessage());
                        }
                        break;
                    }
                }
                Log.d(TAG, "AcceptThread.run: state is " + mState);
                if (mState == STATE_CONNECTED)
                	sendToBTAccessory();
            }
        }
    }
 

    public void cancel() {
    	Log.d(TAG, "Canceling " + this + " Socket Type " + mSocketType); 
        try {
        	if (mmServerSocket != null)
        		mmServerSocket.close();
        } catch (IOException e) {
            Log.e(TAG, "AcceptThread.cancel: Socket Type" + mSocketType + "close() of server failed" + e.getMessage());
        }
    }
}




/**
 * This thread runs during a connection with a remote device.
 * It handles all incoming and outgoing transmissions.
 */
private class ConnectedThread extends Thread {
	static private final String TAG = "PhoneInfoConnectedThread";
    private final BluetoothSocket mmSocket;
    private final InputStream mmInStream;
    private final OutputStream mmOutStream;


    public ConnectedThread(BluetoothSocket socket, String socketType) {
        Log.d(TAG, "create ConnectedThread: " + socketType);
        mmSocket = socket;
        InputStream tmpIn = null;
        OutputStream tmpOut = null;

        // Get the BluetoothSocket input and output streams
        try {
            tmpIn = socket.getInputStream();
            tmpOut = socket.getOutputStream();
            Log.d(TAG, "In and out streams created");
        } catch (IOException e) {
            Log.e(TAG, "temp sockets not created " + e.getMessage());
        }

        mmInStream = tmpIn;
        mmOutStream = tmpOut;
    }


    // this is where we will spend out time when connected to the accessory.
    // When we start, the Accessory will respond to the 'i' query send to it
    // from the connected() method of the service, but afterwards, the connected
    // thread will be blocked on the call for read() that should not return anything
    // as the accessory is not sending any data. Note the the Write will be called
    // though to update the accessory on any change to information
    public void run() {
        Log.d(TAG, "ConnectedThread.run");

        if (mmInStream == null) {
        	Log.e(TAG, "ConnectedThread.run: no InStream");
        	return;        	
        }
        // Keep listening to the InputStream while connected
        while (true) {
            try {
            	Log.d(TAG, "ConnectedThread.run: Reading from InStream socket");
                // Read from the InputStream
            	while (true) {
            		int inB;
            		if (mAccessoryInfo.isAccessoryInfoAvailable()) {
            			// We suppose to get here only once, after receiving all data from
            			// the accessory reply to the 'i' query sent to it by 'connected' method
            			Log.d(TAG, "Accessory display type:" +
            						mAccessoryInfo.getAccessoryType() +
            						" H=" + mAccessoryInfo.getAccessoryHeight() +
            						"W=" + mAccessoryInfo.getAccessoryWidth() );
            			sendDisplayIntensityToActivity(mAccessoryInfo.getAccessoryIntensity());
            			sendBarNotification();
            		}
            		inB = mmInStream.read();
            		if (inB != -1)
            			mAccessoryInfo.addCharacter((byte)inB);
            	}
            } catch (IOException e) {
                Log.e(TAG, "InStream read exception, disconnected " + e.getMessage());
                connectionLost();
                break;
            }
        }
    }
    
    
    /**
     * Write to the connected OutStream.
     * @param buffer  The bytes to write
     */
    public void write(byte[] buffer) {
    	if (mmOutStream == null) {
    		Log.e(TAG, "ConnectedThread.write: no OutStream");
    		return;
    	}
        try {
        	Log.d(TAG, "ConnectedThread.write: writing " + buffer.length + " bytes");
            mmOutStream.write(buffer);

            // Share the sent message back to the UI Activity
            //mHandler.obtainMessage(PhoneInfoActivity.MESSAGE_WRITE, -1, -1, buffer).sendToTarget();
            Log.d(TAG, "ConnectedThread.write: sent to calling activity");
        } catch (IOException e) {
            Log.e(TAG, "Exception during write" + e.getMessage());
        }
    }


    public void cancel() {
        try {
        	Log.d(TAG, "ConnectedThread.cancel: closing socket");
        	if (mmSocket != null)
        		mmSocket.close();
        } catch (IOException e) {
            Log.e(TAG, "ConnectedThread.cancel: socket.close() failed" + e.getMessage());
        }
    }
}



} 